/* Error Message */

#include	<stdio.h>


//------Error String------
char* pszErrString[] =
{
/*	0	*/	"\t\x1b[34m....Done with No-problems....\x1b[0m\n",
/*	1	*/	"\x1b[33mYou should be 'root' to run this program.\x1b[0m\n",

/*	2	*/	"\x1b[33mThere are no Space to store data.\x1b[0m\n",
/*	3	*/	"\x1b[33mExecution faild...\x1b[0m\n",
/*	4	*/	"\x1b[33mComparison faild\x1b[0m\n",

/*	5	*/	"\x1b[33mChcek FAILED.....\x1b[0m\n",
/*	4	*/	"\x1b[33mRx Timeout occured.\x1b[0m\n"
};
